#include "matrix.h"

namespace m2if {

Matrix::Matrix(int nr, int nc)
{
  if ((nr <= 0) || (nc <= 0))
    throw "wrong dimension";

  nr_ = nr;
  nc_ = nc;

  data_ = new double*[nr_];
  if (!data_)
    throw "memory allocation error";

  for (int i = 0; i < nr_; i++)
  {
    data_[i] = new double[nc_];
    if (!data_[i])
    {
      for (int j = 0; j < i; j++)
	delete [] data_[j];
      delete [] data_;
      throw "memory allocation error";
    }
  }
}

Matrix::Matrix(const Matrix & m)
{
  nr_ = m.nr_;
  nc_ = m.nc_;

  data_ = new double*[nr_];
  if (!data_)
    throw "memory allocation error";

  for (int i = 0; i < nr_; i++)
  {
    data_[i] = new double[nc_];
    if (!data_[i])
    {
      for (int j = 0; j < i; j++)
	delete [] data_[j];
      delete [] data_;
      throw "memory allocation error";
    }
    for (int j = 0; j < nc_; j++)
      data_[i][j] = m.data_[i][j];
  }
}

Matrix::~Matrix()
{
  for (int i = 0; i < nr_; i++)
    delete [] data_[i];
  delete [] data_;
}

double & Matrix::operator()(int r, int c)
{
  if ((r < 0) || (r >= nr_) || (c < 0) || (c >= nc_))
    throw "wrong index";
  return data_[r][c];
}

double Matrix::operator()(int r, int c) const
{
  if ((r < 0) || (r >= nr_) || (c < 0) || (c >= nc_))
    throw "wrong index";
  return data_[r][c];
}

Matrix & Matrix::operator=(const Matrix & m)
{
  if (this != &m)
  {
    Matrix tmp = m;
    // we swap tmp.nr_ with nr_
    int i = tmp.nr_;
    tmp.nr_ = nr_;
    nr_ = i;
    // we swap tmp.nc_ with nc_
    i = tmp.nc_;
    tmp.nc_ = nc_;
    nc_ = i;
    // we swap tmp.data_ with data_
    double **d = tmp.data_;
    tmp.data_ = data_;
    data_ = d;
  }

  return *this;
}

std::ostream & operator<<(std::ostream & st, const Matrix & m)
{
  for (int i = 0; i < m.get_nr(); i++)
  {
    for (int j = 0; j < m.get_nc(); j++)
      st << m(i, j) << " ";
    st << "\n";
  }
  return st;
}

Matrix operator+(const Matrix & m1, const Matrix & m2)
{
  if ((m1.get_nr() != m2.get_nr()) || (m1.get_nc() != m2.get_nc()))
    throw "wrong size";

  Matrix m(m1.get_nr(), m1.get_nc());

  for (int i = 0; i < m.get_nr(); i++)
    for (int j = 0; j < m.get_nc(); j++)
      m(i, j) = m1(i, j) + m2(i, j);

  return m;
}

Matrix operator-(const Matrix & m1, const Matrix & m2)
{
  if ((m1.get_nr() != m2.get_nr()) || (m1.get_nc() != m2.get_nc()))
    throw "wrong size";

  Matrix m(m1.get_nr(), m1.get_nc());

  for (int i = 0; i < m.get_nr(); i++)
    for (int j = 0; j < m.get_nc(); j++)
      m(i, j) = m1(i, j) - m2(i, j);

  return m;
}

Matrix operator*(const Matrix & m1, const Matrix & m2)
{
  if (m1.get_nc() != m2.get_nr())
    throw "wrong size";

  Matrix m(m1.get_nr(), m2.get_nc());

  for (int i = 0; i < m.get_nr(); i++)
    for (int j = 0; j < m.get_nc(); j++)
    {
      m(i, j) = 0;
      for (int k = 0; k < m1.get_nc(); k++)
	m(i, j) += m1(i, k) * m2(k, j);
    }

  return m;
}

Matrix operator*(const Matrix & m, const double & v)
{
  Matrix res(m.get_nr(), m.get_nc());

  for (int i = 0; i < m.get_nr(); i++)
    for (int j = 0; j < m.get_nc(); j++)
      res(i, j) = m(i, j) * v;

  return m;
}

Matrix operator*( const double & v, const Matrix & m)
{
  return m * v;
}

} // namespace m2if
